@@ -2,47 +2,44 @@ r"""
 This module aims to provide the core components that are required to build
 a Parflow input deck.
 """
-import sys
 import yaml
-
-try:
-    from yaml import CDumper as YAMLDumper
-except ImportError:
-    from yaml import Dumper as YAMLDumper
+import sys
 
 from parflow.tools import settings
 from parflow.tools.fs import get_text_file_content
-from parflow.tools.helper import map_to_child, map_to_children_of_type, map_to_parent, map_to_self
-from parflow.tools.helper import sort_dict_by_priority
+from parflow.tools.helper import (
+    map_to_child, map_to_children_of_type, map_to_parent, map_to_self,
+    remove_prefix
+)
+from parflow.tools.helper import normalize_location, sort_dict_by_priority
 from parflow.tools.io import read_pfidb
 
 from .domains import validate_value_to_string, validate_value_with_exception
 from .handlers import decorate_value
 
+
 # -----------------------------------------------------------------------------
 # Accessor helpers
-# -----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
-def validate_helper(container_obj, name, value, indent):
+def validate_helper(container, name, value, indent):
     """Helper function for validating a value
     """
-    nbErrors = 0
+    num_errors = 0
     validation_string = ''
-    has_default = True if 'default' in container_obj._details_[name] else False
-    history = None
-    if 'history' in container_obj._details_[name] and len(container_obj._details_[name]['history']):
-        history = container_obj._details_[name]['history']
-    if 'default' in container_obj._details_[name] \
-            and value == container_obj._details_[name]['default'] \
-            and 'MandatoryValue' not in container_obj._details_[name]['domains']:
+    details = container._details_[name]
+    has_default = 'default' in details
+    history = details.get('history')
+    if (has_default and value == details['default'] and
+            'MandatoryValue' not in details['domains']):
         pass
     else:
-        nbErrors, validation_string = \
+        num_errors, validation_string = \
             validate_value_to_string(
-                container_obj, value, has_default, container_obj._details_[name]['domains'],
-                container_obj.get_context_settings(), history, indent)
+                container, value, has_default, details['domains'],
+                container.get_context_settings(), history, indent)
 
-    return nbErrors, validation_string
+    return num_errors, validation_string
 
 # -----------------------------------------------------------------------------
 
@@ -50,30 +47,19 @@ def validate_helper(container_obj, name, value, indent):
 def detail_helper(container, name, value):
     """Helper function that extract elements of the field's detail
     """
-    domains = None
-    handlers = None
-    history = None
-    crosscheck = None
-    if name in container._details_:
-        if 'domains' in container._details_[name]:
-            domains = container._details_[name]['domains']
-
-        if 'handlers' in container._details_[name]:
-            handlers = container._details_[name]['handlers']
-
-        if 'history' in container._details_[name]:
-            history = container._details_[name]['history']
-
-        else:
-            history = []
-            container._details_[name]['history'] = history
+    details = container._details_.get(name, {})
+    domains = details.get('domains')
+    handlers = details.get('handlers')
+    crosscheck = details.get('crosscheck')
+    if details:
+        history = details.setdefault('history', [])
         history.append(value)
-
-        if 'crosscheck' in container._details_[name]:
-            crosscheck = container._details_[name]['crosscheck']
+    else:
+        history = None
 
     return domains, handlers, history, crosscheck
 
+
 # -----------------------------------------------------------------------------
 # Internal field name helpers
 # -----------------------------------------------------------------------------
@@ -85,6 +71,7 @@ def is_private_key(name):
     """
     return name[0] == '_' and name[-1] == '_'
 
+
 # -----------------------------------------------------------------------------
 
 def is_not_private_key(name):
@@ -94,9 +81,10 @@ def is_not_private_key(name):
     """
     return not is_private_key(name)
 
+
 # -----------------------------------------------------------------------------
 
-def convert_value_for_string_dict(value):
+def to_str_dict_format(value):
     """Ensure that the output value is a valid string
     """
     if isinstance(value, str):
@@ -107,6 +95,7 @@ def convert_value_for_string_dict(value):
 
     return value
 
+
 # -----------------------------------------------------------------------------
 
 def extract_keys_from_object(dict_to_fill, instance, parent_namespace=''):
@@ -115,65 +104,60 @@ def extract_keys_from_object(dict_to_fill, instance, parent_namespace=''):
     """
     if hasattr(instance, '_pfstore_'):
         for key, value in instance._pfstore_.items():
-            dict_to_fill[key] = convert_value_for_string_dict(value)
+            dict_to_fill[key] = to_str_dict_format(value)
 
-    for key in instance.get_key_names(skip_default=True):
+    for key in instance.keys(skip_default=True):
         value = instance[key]
         if value is None:
             continue
 
-        full_qualified_key = instance.get_parflow_key(parent_namespace, key)
+        full_qualified_key = instance.to_pf_name(parent_namespace, key)
         if isinstance(value, PFDBObj):
             if hasattr(value, '_value_'):
                 has_details = hasattr(value, '_details_') \
                     and '_value_' in value._details_
-                has_default = has_details \
-                    and 'default' in value._details_['_value_']
-                has_domain = has_details \
-                    and 'domains' in value._details_['_value_']
+                details = value._details_['_value_'] if has_details else None
+                has_default = has_details and 'default' in details
+                has_domain = has_details and 'domains' in details
                 is_mandatory = has_domain \
-                    and 'MandatoryValue' in value._details_['_value_']['domains']
-                is_default = has_default \
-                    and value._value_ == value._details_['_value_']['default']
-                is_set = has_details \
-                    and 'history' in value._details_['_value_'] \
-                    and len(value._details_['_value_']['history']) > 0
-
+                    and 'MandatoryValue' in details['domains']
+                is_default = has_default and \
+                    value._value_ == details['default']
+                is_set = has_details and details.get('history')
                 if is_mandatory or not is_default or is_set:
                     dict_to_fill[full_qualified_key] = \
-                        convert_value_for_string_dict(value._value_)
+                        to_str_dict_format(value._value_)
             extract_keys_from_object(dict_to_fill, value, full_qualified_key)
         else:
-            dict_to_fill[full_qualified_key] = \
-                convert_value_for_string_dict(value)
+            dict_to_fill[full_qualified_key] = to_str_dict_format(value)
+
 
 # -----------------------------------------------------------------------------
 
-def extract_keys_from_dict(dict_to_fill, dictObj, parent_namespace=''):
+def extract_keys_from_dict(dict_to_fill, dict_obj, parent_namespace=''):
     """Helper function to extract a flat key/value dictionary for
     a given PFDBObj inside dict_to_fill.
     """
-    for key, value in dictObj.items():
-        if len(parent_namespace) and key == '$_' or key == '_value_':
+    for key, value in dict_obj.items():
+        if parent_namespace and key == '_value_':
             dict_to_fill[parent_namespace] = value
             continue
 
         if value is None or is_private_key(key):
             continue
 
-        full_qualified_key = f'{parent_namespace}.{key}' if parent_namespace else key
+        full_qualified_key = (
+            f'{parent_namespace}.{key}' if parent_namespace else key
+        )
         if isinstance(value, dict):
-            # Need to handle _value_ and $_
+            # Need to handle _value_
             if hasattr(value, '_value_'):
-                dict_to_fill[full_qualified_key] = convert_value_for_string_dict(
-                    value._value_)
-            if hasattr(value, '$_'):
-                dict_to_fill[full_qualified_key] = convert_value_for_string_dict(
+                dict_to_fill[full_qualified_key] = to_str_dict_format(
                     value._value_)
             extract_keys_from_dict(dict_to_fill, value, full_qualified_key)
         else:
-            dict_to_fill[full_qualified_key] = convert_value_for_string_dict(
-                value)
+            dict_to_fill[full_qualified_key] = to_str_dict_format(value)
+
 
 # -----------------------------------------------------------------------------
 
@@ -185,6 +169,7 @@ def flatten_hierarchical_map(hierarchical_map):
     extract_keys_from_dict(flat_map, hierarchical_map, parent_namespace='')
     return flat_map
 
+
 # -----------------------------------------------------------------------------
 # Main DB Object
 # -----------------------------------------------------------------------------
@@ -199,13 +184,13 @@ class PFDBObj:
         self._parent_ = parent
         self._prefix_ = None
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
     def __setitem__(self, key, value):
         """Allow a[x] for assignment as well"""
         self.__setattr__(key, value)
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
     def __setattr__(self, name, value):
         """
@@ -219,18 +204,19 @@ class PFDBObj:
             if name in self._details_:
                 domains, handlers, history, crosscheck = detail_helper(
                     self, name, value)
-            elif hasattr(self, name) and isinstance(self.__dict__[name], PFDBObj):
+            elif hasattr(self, name) and isinstance(self.__dict__[name],
+                                                    PFDBObj):
                 # Handle value object assignment
                 value_object_assignment = True
                 value_obj = self.__dict__[name]
                 domains, handlers, history, crosscheck = detail_helper(
                     value_obj, '_value_', value)
             else:
-                print(
-                    f'Field {name} is not part of the expected schema {self.__class__}')
+                msg = (f'Field {name} is not part of the expected '
+                       f'schema {self.__class__}')
+                print(msg)
                 if settings.EXIT_ON_ERROR:
-                    raise ValueError(
-                        f'Field "{name}" is not part of the expected schema {self.__class__}')
+                    raise ValueError(msg)
 
         # Run domain validation
         if settings.PRINT_LINE_ERROR:
@@ -244,7 +230,7 @@ class PFDBObj:
             # Decorate value if need be (i.e. Geom.names: 'a b c')
             self.__dict__[name] = decorate_value(value, self, handlers)
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
     def __len__(self):
         """
@@ -258,7 +244,7 @@ class PFDBObj:
         if hasattr(self, '_value_') and self._value_ is not None:
             value_count += 1
 
-        for name in self.get_key_names(True):
+        for name in self.keys(True):
             obj = self.__dict__[name]
             if isinstance(obj, PFDBObj):
                 value_count += len(obj)
@@ -271,7 +257,7 @@ class PFDBObj:
 
         return value_count
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
     def __getitem__(self, key):
         """
@@ -290,40 +276,15 @@ class PFDBObj:
             prefix = self._details_['_prefix_']
 
         key_str = f'{prefix}{key_str}'
-        if hasattr(self, key_str):
-            return getattr(self, key_str)
+        if not hasattr(self, key_str):
+            print(f'Could not find key {key}/{key_str} in '
+                  f'{list(self.__dict__.keys())}')
 
-        print(f'Could not find key {key}/{key_str} in {self.__dict__.keys()}')
         return getattr(self, key_str)
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def help(self, key=None):
-        """
-        Dynamic help function for runtime evaluation
-        """
-        if key is not None:
-            if key in self._details_:
-                if 'help' in self._details_[key]:
-                    print(self._details_[key]['help'])
-            else:
-                obj = self.__dict__[key]
-                if hasattr(obj, '__doc__'):
-                    print(obj.__doc__)
-
-                if hasattr(obj, '_details_') and '_value_' in obj._details_ \
-                        and 'help' in obj._details_['_value_']:
-                    print(obj._details_['_value_']['help'])
-
-        elif hasattr(self, '__doc__'):
-            print(self.__doc__)
-            if hasattr(self, '_details_') and '_value_' in self._details_ \
-                    and 'help' in self._details_['_value_']:
-                print(self._details_['_value_']['help'])
-
-    # ---------------------------------------------------------------------------
-
-    def get_key_dict(self):
+    def to_dict(self):
         """Method that will return a flat map of all the ParFlow keys.
 
         Returns:
@@ -334,9 +295,9 @@ class PFDBObj:
         extract_keys_from_object(key_dict, self)
         return key_dict
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def get_key_names(self, skip_default=False):
+    def keys(self, skip_default=False):
         """
         Gets the key names necessary for the run while skiping unset ones
         """
@@ -356,17 +317,13 @@ class PFDBObj:
             else:
                 has_details = hasattr(self, '_details_') \
                     and name in self._details_
-                has_default = has_details \
-                    and 'default' in self._details_[name]
-                has_domain = has_details \
-                    and 'domains' in self._details_[name]
+                details = self._details_[name] if has_details else None
+                has_default = has_details and 'default' in details
+                has_domain = has_details and 'domains' in details
                 is_mandatory = has_domain \
-                    and 'MandatoryValue' in self._details_[name]['domains']
-                is_default = has_default \
-                    and obj == self._details_[name]['default']
-                is_set = has_details \
-                    and 'history' in self._details_[name] \
-                    and len(self._details_[name]['history']) > 0
+                    and 'MandatoryValue' in details['domains']
+                is_default = has_default and obj == details['default']
+                is_set = has_details and details.get('history')
 
                 if obj is not None:
                     if skip_default:
@@ -378,18 +335,24 @@ class PFDBObj:
                 elif is_mandatory:
                     yield name
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def validate(self, indent=1, skip_valid=False, enable_print=True):
+    def validate(self, indent=1, verbose=False, enable_print=True,
+                 working_directory=None):
         """
         Method to validate sub hierarchy
         """
         if len(self) == 0:
             return 0
 
+        # overwrite current working directory
+        prev_dir = settings.WORKING_DIRECTORY
+        if working_directory:
+            settings.set_working_directory(working_directory)
+
         error_count = 0
         indent_str = '  '*indent
-        for name in self.get_key_names(skip_default=True):
+        for name in self.keys(skip_default=True):
             obj = self.__dict__[name]
             if isinstance(obj, PFDBObj):
                 if len(obj):
@@ -398,33 +361,36 @@ class PFDBObj:
                         add_errors, validation_string = validate_helper(
                             obj, '_value_', value, indent)
 
-                        if enable_print and (add_errors or not skip_valid):
+                        if enable_print and (add_errors or verbose):
                             print(f'{indent_str}{name}: {validation_string}')
 
                         error_count += add_errors
                     elif enable_print:
-                        if not skip_valid or obj.validate(enable_print=False):
+                        if verbose or obj.validate(enable_print=False):
                             print(f'{indent_str}{name}:')
 
                     error_count += obj.validate(indent + 1,
-                                                skip_valid=skip_valid,
+                                                verbose=verbose,
                                                 enable_print=enable_print)
 
             elif hasattr(self, '_details_') and name in self._details_:
                 add_errors, validation_string = validate_helper(
                     self, name, obj, indent)
-                if enable_print and (not skip_valid or add_errors):
+                if enable_print and (verbose or add_errors):
                     print(f'{indent_str}{name}: {validation_string}')
                 error_count += add_errors
             elif obj is not None:
-                if enable_print and not skip_valid:
+                if enable_print and verbose:
                     print(f'{indent_str}{name}: {obj}')
 
+        # revert working directory to original directory
+        settings.set_working_directory(prev_dir)
+
         return error_count
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def get_full_key_name(self):
+    def full_name(self):
         """
         Helper method returning the full name of a given ParFlow key.
         """
@@ -436,11 +402,9 @@ class PFDBObj:
             parent = current_location._parent_
             for name in parent.__dict__:
                 value = parent.__dict__[name]
-                if value == current_location:
-                    if current_location._prefix_:
-                        full_path.append(name[len(current_location._prefix_):])
-                    else:
-                        full_path.append(name)
+                if value is current_location:
+                    prefix = current_location._prefix_
+                    full_path.append(remove_prefix(name, prefix))
             current_location = parent
             if count > len(full_path):
                 return f'not found {count}: {".".join(full_path)}'
@@ -448,30 +412,28 @@ class PFDBObj:
         full_path.reverse()
         return '.'.join(full_path)
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def get_parflow_key(self, parent_namespace, key):
+    def to_pf_name(self, parent_namespace, key):
         """
-        Helper method returning the key to use for Parflow on a given field key.
-        This allow to handle differences between what can be defined in Python vs Parflow key.
+        Helper method returning the key to use for Parflow on a given
+        field key. This allows us to handle differences between what
+        can be defined in Python vs Parflow key.
         """
         value = self.__dict__[key]
         prefix = ''
         if isinstance(value, PFDBObj):
             if value._prefix_ and key.startswith(value._prefix_):
                 prefix = value._prefix_
-        else:
-            if key in self._details_:
-                detail = self._details_[key]
-                if '_prefix_' in detail:
-                    prefix = detail["_prefix_"]
+        elif key in self._details_:
+            detail = self._details_[key]
+            if '_prefix_' in detail:
+                prefix = detail['_prefix_']
 
-        if parent_namespace:
-            return f'{parent_namespace}.{key[len(prefix):]}'
+        start = f'{parent_namespace}.' if parent_namespace else ''
+        return start + remove_prefix(key, prefix)
 
-        return key[len(prefix):]
-
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
     def get_children_of_type(self, class_name):
         """Return a list of PFDBObj of a given type that are part of
@@ -486,27 +448,28 @@ class PFDBObj:
 
         return results
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def get_selection_from_location(self, location='.'):
+    @normalize_location
+    def select(self, location='.'):
         """
         Return a PFDBObj object based on a location.
 
         i.e.:
-          run.Process.Topology.get_selection_from_location('.') => run.Process.Topology
-          run.Process.Topology.get_selection_from_location('..') => run.Process
-          run.Process.Topology.get_selection_from_location('../../Geom') => run.Geom
-          run.Process.Topology.get_selection_from_location('/Geom') => run.Geom
+          run.Process.Topology.select('.') => run.Process.Topology
+          run.Process.Topology.select('..') => run.Process
+          run.Process.Topology.select('../../Geom') => run.Geom
+          run.Process.Topology.select('/Geom') => run.Geom
         """
         current_location = self
         path_items = location.split('/')
-        if location[0] == '/':
+        if location.startswith('/'):
             while current_location._parent_ is not None:
                 current_location = current_location._parent_
 
         next_list = [current_location]
         for path_item in path_items:
-            if path_item == '':
+            if not path_item:
                 continue
 
             current_list = next_list
@@ -516,19 +479,82 @@ class PFDBObj:
                 next_list.extend(map(map_to_parent, current_list))
             elif path_item == '.':
                 next_list.extend(map(map_to_self, current_list))
-            elif path_item[0] == '{':
-                multiList = map(map_to_children_of_type(
+            elif path_item.startswith('{'):
+                multi_list = map(map_to_children_of_type(
                     path_item[1:-1]), current_list)
-                next_list = [item for sublist in multiList for item in sublist]
+                next_list = [x for sublist in multi_list for x in sublist]
             else:
                 next_list.extend(map(map_to_child(path_item), current_list))
-                if len(next_list) and isinstance(next_list[0], list):
-                    next_list = [
-                        item for sublist in next_list for item in sublist]
+                if next_list and isinstance(next_list[0], list):
+                    next_list = [x for sublist in next_list for x in sublist]
 
         return next_list
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
+
+    @normalize_location
+    def value(self, location='.', skip_default=False):
+        """
+        Return a value based on a location.
+
+        e.g.:
+          run.ComputationalGrid.value('DX') => 8.88
+          run.value('ComputationalGrid/DX') => 8.88
+          run.value('/ComputationalGrid/DX') => 8.88
+          run.Perm.value('../ComputationalGrid/DX') => 8.88
+          run.Solver.value() => 'Richards'
+        """
+        value, *_ = self._value(location)
+
+        if skip_default:
+            details = self.details(location)
+            if not details.get('history'):
+                return
+
+        return value
+
+    # -------------------------------------------------------------------------
+
+    @normalize_location
+    def details(self, location='.'):
+        """
+        Return details based on a location.
+        """
+        value, container, key = self._value(location)
+
+        if key and isinstance(getattr(container, key, None), PFDBObj):
+            value = container[key]
+
+        if isinstance(value, PFDBObj):
+            return getattr(value, '_details_', {}).get('_value_', {})
+
+        key = '_value_' if key is None else key
+        return getattr(container, '_details_', {}).get(key, {})
+
+    # -------------------------------------------------------------------------
+
+    @normalize_location
+    def doc(self, location='.'):
+        """
+        Return docs based on a location.
+        """
+        value, container, key = self._value(location)
+        details = self.details(location)
+
+        if key and isinstance(getattr(container, key, None), PFDBObj):
+            value = container[key]
+
+        if not value and isinstance(container, PFDBObj):
+            value = container
+
+        ret = ''
+        if isinstance(value, PFDBObj) and getattr(value, '__doc__', None):
+            ret += value.__doc__ + '\n'
+        if details.get('help'):
+            ret += details['help'] + '\n'
+        return ret
+
+    # -------------------------------------------------------------------------
 
     def get_context_settings(self):
         """
@@ -542,33 +568,33 @@ class PFDBObj:
             'pf_version': settings.PARFLOW_VERSION
         }
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def pfset(self, key='', value=None, yamlFile=None, yamlContent=None,
-              pfidbFile=None, hierarchical_map=None, flat_map=None,
+    def pfset(self, key='', value=None, yaml_file=None, yaml_content=None,
+              pfidb_file=None, hierarchical_map=None, flat_map=None,
               exit_if_undefined=False):
         """
         Allow to define any parflow key so it can be exported.
         Many formats are supported:
             - key/value: To set a single value relative to our current
                 PFDBObj.
-            - yamlFile: YAML file path to load and import using the
+            - yaml_file: YAML file path to load and import using the
                 current PFDBObj as root.
-            - yamlContent: YAML string to load and import using the
+            - yaml_content: YAML string to load and import using the
                 current PFDBObj as root.
             - hierarchical_map: Nested dict containing several key/value
                 pair using the current PFDBObj as root.
             - flat_map: Flat dict with parflow key/value pair to set
                 using the current PFDBObj as root.
         """
-        if yamlFile:
-            yamlContent = get_text_file_content(yamlFile)
+        if yaml_file:
+            yaml_content = get_text_file_content(yaml_file)
 
-        if yamlContent:
-            hierarchical_map = yaml.safe_load(yamlContent)
+        if yaml_content:
+            hierarchical_map = yaml.safe_load(yaml_content)
 
-        if pfidbFile:
-            flat_map = read_pfidb(pfidbFile)
+        if pfidb_file:
+            flat_map = read_pfidb(pfidb_file)
 
         if hierarchical_map:
             flat_map = flatten_hierarchical_map(hierarchical_map)
@@ -585,8 +611,7 @@ class PFDBObj:
         key_stored = False
         tokens = key.split('.')
         if len(tokens) > 1:
-            container = self.get_selection_from_location(
-                '/'.join(tokens[:-1]))[0]
+            container, = self.select('/'.join(tokens[:-1]))
             if container is not None:
                 container[tokens[-1]] = value
                 key_stored = True
@@ -598,74 +623,80 @@ class PFDBObj:
             # store key on the side
             if '_pfstore_' not in self.__dict__:
                 self.__dict__['_pfstore_'] = {}
-            parentNamespace = self.get_full_key_name()
-            fullkeyName = f"{parentNamespace}" \
-                          f"{'.' if parentNamespace else ''}{key}"
-            self.__dict__['_pfstore_'][fullkeyName] = value
-            rootPath = self.get_full_key_name()
+            parent_namespace = self.full_name()
+            full_key_name = f"{parent_namespace}" \
+                            f"{'.' if parent_namespace else ''}{key}"
+            self.__dict__['_pfstore_'][full_key_name] = value
+            root_path = self.full_name()
             print(f"Caution: Using internal store of "
-                  f"{rootPath if rootPath else 'run'} "
-                  f"to save {fullkeyName} = {value}")
+                  f"{root_path if root_path else 'run'} "
+                  f"to save {full_key_name} = {value}")
             if exit_if_undefined:
                 sys.exit(1)
 
-    # ---------------------------------------------------------------------------
+    # -------------------------------------------------------------------------
 
-    def get(self, key='', skip_default=False):
-        value = None
-        tokens = key.split('.')
-        details = None
-        if len(tokens) > 1:
-            container = self.get_selection_from_location(
-                '/'.join(tokens[:-1]))[0]
-            if container is not None:
-                value = container[tokens[-1]] if \
-                    tokens[-1] in container.__dict__ else None
-            if value is not None and not isinstance(value, PFDBObj):
-                details = container._details_[tokens[-1]]
-        elif len(tokens) == 1:
-            if len(tokens[0]) > 0:
-                value = self[tokens[0]] if tokens[0] in self.__dict__ else None
-                details = self._details_[tokens[0]] if \
-                    tokens[0] in self._details_ else None
-            else:
-                value = self
-
-        if value is None:
-            return None
-
-        if isinstance(value, PFDBObj):
-            if skip_default:
-                details = value._details_['_value_']
-                if 'history' not in details or len(details['history']) == 0:
-                    return None
-            return value._value_
-
-        if skip_default and details:
-            if 'history' not in details or len(details['history']) == 0:
-                return None
-
-        return value
-
-    # ---------------------------------------------------------------------------
-
-    def process_dynamic(self):
+    def _process_dynamic(self):
         """
         Processing the dynamically defined (user-defined) key names
         """
         from . import generated
-        for (class_name, selection) in self._dynamic_.items():
+        for class_name, selection in self._dynamic_.items():
             klass = getattr(generated, class_name)
-            names = self.get_selection_from_location(selection)
+            names = self.select(selection)
             for name in names:
                 if name is not None:
                     self.__dict__[name] = klass(self)
 
+    # -------------------------------------------------------------------------
+
+    @normalize_location
+    def _value(self, location='.'):
+        """
+        Internal function to get the value, container and key from location
+        """
+        container, key = self._get_container_and_key(location)
+
+        if key is None:
+            value = container
+        else:
+            value = getattr(container, key, None)
+
+        if isinstance(value, PFDBObj):
+            value = getattr(value, '_value_', None)
+
+        return value, container, key
+
+    # -------------------------------------------------------------------------
+
+    @normalize_location
+    def _get_container_and_key(self, location):
+        """
+        Internal function to get the container and key from location
+        """
+        split = location.split('/')
+        path, key = '/'.join(split[:-1]), split[-1]
+
+        if key == '.':
+            return self, None
+
+        parent = self
+        while key == '..' and getattr(parent, '_parent_', None):
+            parent = parent._parent_
+            split = path.split('/')
+            path, key = '/'.join(split[:-1]), split[-1]
+
+        if parent is not self:
+            # We went through the loop above. Return...
+            return parent, None
+
+        return self.select(path)[0], key
+
+
 # -----------------------------------------------------------------------------
 # Main DB Object
 # -----------------------------------------------------------------------------
 
-
 class PFDBObjListNumber(PFDBObj):
     """Class for leaf list values"""
 
@@ -686,14 +717,10 @@ class PFDBObjListNumber(PFDBObj):
 
         self.__dict__[key_str] = value
 
-    def get_parflow_key(self, parent_namespace, key):
+    def to_pf_name(self, parent_namespace, key):
         """Helper method returning the key to use for Parflow on
         a given field key. This allows handling of differences
         between what can be defined in Python vs Parflow key.
         """
-        prefix = self._prefix_ if self._prefix_ else ''
-
-        if parent_namespace:
-            return f'{parent_namespace}.{key[len(prefix):]}'
-
-        return key[len(prefix):]
+        start = f'{parent_namespace}.' if parent_namespace else ''
+        return start + remove_prefix(key, self._prefix_)
